<?php
/**
 * @file
 * Organic group title field and widget functions.
 */


/**
 * Implements hook_field_info().
 */
function og_titles_field_info() {
  return array(
    'og_member_title' => array(
      'label' => t('Group member title'),
      'description' => t('A field attached to group membership for user entities.'),
      'settings' => array('max_length' => 255),
      'instance settings' => array('text processing' => FALSE),
      'default_widget' => 'text_textfield',
      'default_formatter' => 'text_plain',
    ),
  );
}

/**
 * Implements hook_field_formatter_info().
 */
function og_titles_field_formatter_info() {
  return array(
    'og_titles_default_format' => array(
      'label' => t('Default text format'),
      'field types' => array('og_member_title'),
      'settings' => array(
        'format' => filter_default_format(),
      ),
    ),
  );
}

/**
 * Implements hook_field_formatter_info_alter().
 */
function og_titles_field_formatter_info_alter(&$info) {
  $info['text_plain']['field types'][] = 'og_member_title';
}

/**
 * Implements hook_field_widget_info().
 */
function og_titles_field_widget_info() {
  return array(
    'og_titles_select' => array(
      'label' => t('Group member title select'),
      'description' => t('Select list widget for group member title field based on a group titles.'),
      'field types' => array('og_member_title'),
      'settings' => array(),
      'behaviors' => array(
        'multiple_values' => FIELD_BEHAVIOR_DEFAULT,
        'default_value' => FIELD_BEHAVIOR_DEFAULT,
      ),
    ),
  );
}

/**
 * Implements hook_field_widget_info_alter().
 */
function og_titles_field_widget_info_alter(&$info) {

  // Add title field to select and text input field widgets.
  $info['text_textfield']['field types'][] = 'og_member_title';

  // @todo find other possibilities

}

/**
 * Implements hook_field_validate().
 */
function og_titles_field_validate($entity_type, $entity, $field, $instance, $lang, $items, &$errors) {
  if (!isset($entity)) {
    // Field configuration.
    return;
  }

  $group = og_entity_getter($entity, array(), 'group');
  $group_node = $group->value();

  if (isset($group_node->group_titles[$lang])) {
    // Validate based on group title values.
    $allowed = array_map(function($value) {
      return $value['value'];
    }, $group_node->group_titles['und']);

    $value = is_string($items[0]) ? $items[0] : $items[0]['value'];
    if (!in_array($value, $allowed)) {
      $errors[$lang][] = t('Illegal value for member title, %title.', array('%title' => $value));
    }
  }
}

/**
 * Implements hook_field_presave().
 */
function og_titles_field_presave($entity_type, $entity, $field, $instance, $lang, &$items) {
  if (!isset($entity)) {
    // Field configuration.
    return;
  }

  if (empty($items)) {
    return;
  }

  if (is_string($items[0])) {
    // Sometimes it's a string.
    $value = $items[0];
    $items[0] = array(
      'value' => $value,
      'weight' => 0,
    );
  }

  $group = og_entity_getter($entity, array(), 'group');
  $group_node = $group->value();

  if (isset($group_node->group_titles[$lang])) {

    foreach (element_children($group_node->group_titles[$lang]) as $delta) {
      if ($group_node->group_titles[$lang][$delta]['value'] == $items[0]['value']) {
        // Determine weight of title if it exists and matches value in group node.
        $items[0]['weight'] = $delta;
      }
    }
  }
  else {
    // Set a default weight of zero.
    $items[0]['weight'] = 0;
  }
}

/**
 * Implements hook_field_is_empty().
 * @param $item the value of the item as a string or an array.
 * @param $field the field.
 * @return boolean
 */
function og_titles_field_is_empty($item, $field) {
  $value = is_string($item) ? $item : $item['value'];

  if (empty($value) && (string) $value !== '0') {
    return TRUE;
  }

  return FALSE;
}

/**
 * Implements hook_field_formatter_settings_form().
 */
function og_titles_field_formatter_settings_form($field, $instance, $view_mode, $form, &$form_state) {
  global $user;
  $display = $instance['display'][$view_mode];
  $settings = $display['settings'];

  $element = array();

  if ($display['type'] == 'og_titles_default_format' && $instance['type'] == 'text_textfield') {
    $formats = filter_formats($user);

    foreach ($formats as $format) {
      $options[$format->format] = check_plain($format->name);
    }

    $element['format'] = array(
      '#type' => 'select',
      '#title' => t('Text format'),
      '#description' => t('Choose the text format to display the field value. You are limited to text formats that you have access to. You should only allow a format you are comfortable with all authenticated users using.'),
      '#options' => $options,
      '#default_value' => $settings['format'],
    );
  }

  return $element;
}

/**
 * Implements hook_field_formatter_settings_summary().
 */
function og_titles_field_formatter_settings_summary($field, $instance, $view_mode) {
  $display = $instance['display'][$view_mode];
  $summary = '';

  if ($display['type'] == 'og_titles_default_format') {
    $summary = t('All titles will be formatted in the %format format.', array('%format' => $display['settings']['format']));
  }

  return $summary;
}

/**
 * Implements hook_field_formatter_view().
 */
function og_titles_field_formatter_view($entity_type, $entity, $field, $instance, $lang, &$items, $display) {
  $element = array();

  if ($display['type'] == 'og_titles_default_format') {
    foreach (element_children($items) as $delta) {
      // Filter value through the text format.
      $element[$delta] = array(
        '#markup' => check_markup($items[$delta]['value'], $display['settings']['format']),
      );
    }
  }

  return $element;
}

/**
 * Implements hook_field_widget_form().
 */
function og_titles_field_widget_form(&$form, &$form_state, $field, $instance, $lang, $items, $delta, $element) {

  if (isset($form_state['og_membership'])) {
    $membership = $form_state['og_membership'];
    $entity = entity_load_single($membership->group_type, $membership->gid);
    $options = array();

    if (isset($entity->group_titles[$lang])) {
      foreach (element_children($entity->group_titles[$lang]) as $n) {
        $options[$n] = check_plain($entity->group_titles[$lang][$n]['value']);
      }
    }

    $reversed = array_flip($options);

    $element += array(
      '#type' => 'select',
      '#options' => $options,
      '#default_value' => !empty($reversed) ? $reversed[$items[$delta]['value']] : '',
      '#element_validate' => array('og_titles_select_list_validate'),
    );
  }
  else {
    $element += array(
      '#type' => 'container',
      '#attributes' => array(
        'class' => array('messages'),
      ),
      'message' => array(
        '#markup' => t('Organic groups title widget not available.'),
      ),
    );
  }

  return $element;
}

/**
 * Select list validate callback
 */
function og_titles_select_list_validate($element, &$form_state) {
  $options = $element['#options'];
  $value = $element['#value'];

  form_set_value($element, $options[$value], $form_state);
}

